// IRP Test Driver Client Program
//
// By Enrico Martignetti


#include <windows.h>
#include <stdio.h>

#include "DrvR3.h"





int wmain( int argC, wchar_t *lpwszArgV, wchar_t *lpwszEnvV) {

	WCHAR	wszDriverName[] = L"IoCompl.sys";
    HANDLE  hDevice = INVALID_HANDLE_VALUE;

	BOOL	bStatus;
	BYTE	buffer[100];
	DWORD	dwBytesRead;
	DWORD	dwLastErr;
	int		iRC;
	OVERLAPPED ov;

	printf("\nTest client, compiled %s %s\n",__DATE__, __TIME__);

	ZeroMemory(&ov, sizeof ov);

	ov.hEvent = CreateEvent(
		NULL,		// Security attrs.
		TRUE,		// Manual reset
		FALSE,		// Not signaled
		NULL		// No name
		);
	if (ov.hEvent == NULL) {
		dwLastErr = GetLastError();
		wprintf(L"\nCreateEvent failed with error 0x%8x, %d", dwLastErr, dwLastErr);
		return 1;
	}

	
    hDevice =
        CreateFile( L"\\\\.\\"DRV_DEVICE_NAME,
					GENERIC_READ | GENERIC_WRITE,
					FILE_SHARE_READ | FILE_SHARE_WRITE,
					NULL,					// no security
					OPEN_EXISTING,
					FILE_FLAG_OVERLAPPED,	// async read
//						0,					// sync read
					NULL );					// no template

	if ( hDevice == INVALID_HANDLE_VALUE ) {
		wprintf( L"Failed to obtain file handle to %s. "
			L"GetLastError() = %d (0x%x)\n",
			DRV_DEVICE_NAME,
			GetLastError(),
			GetLastError());
        return 1;
	}
    wprintf(L"\n%s succesfully opened.", DRV_DEVICE_NAME);
	iRC = 0;

	if (!ReadFile(
		hDevice, 
		buffer, 
		1, 
		&dwBytesRead, 
		&ov				// async read
//		NULL			// sync read
		)) {
		dwLastErr = GetLastError();
		if (dwLastErr == ERROR_IO_PENDING) {
		    wprintf(L"\nReadFile() returned with pending I/O");
			bStatus = GetOverlappedResult(
				hDevice,
				&ov,
				&dwBytesRead,
				TRUE);
			if (bStatus) {
				wprintf(L"\nReadFile() succeeded.");
				wprintf(L"\n%d bytes read.", dwBytesRead);
				goto CLEANUP;
			} else {
				dwLastErr = GetLastError();
				wprintf(L"\nGetOverlappedResult() failed. GetLastError() = %d (0x%8x)",
					dwLastErr,
					dwLastErr);
				iRC = 1;
				goto CLEANUP;
			}
		} else {
			wprintf(L"\nReadFile() failed. GetLastError() = %d (0x%8x)",
				dwLastErr,
				dwLastErr);
			iRC = 1;
			goto CLEANUP;
		}
	} else {
	    wprintf(L"\nReadFile() returned with completed I/O");
		wprintf(L"\nReadFile() succeeded.");
		wprintf(L"\n%d bytes read.", dwBytesRead);
	}

CLEANUP:
	if (ov.hEvent != NULL) CloseHandle(ov.hEvent);
	if (hDevice != INVALID_HANDLE_VALUE) {
		wprintf(L"\nClosing device handle");
		CloseHandle(hDevice);
	}
	wprintf(L"\nLeaving wmain()");
	wprintf(L"\n");
    return iRC;

}


